"""
 You need have 3 files in this script directory:
 1. new.rul ... new-version of original strings (e.g. ver.98D1)
 2. tra.rul ... your translated prev-version (e.g. czech ver.98b)
 3. old.rul ... prev-version of original strings (e.g. ver.98b)

 You can have your own strings at the END of translation separated
 by comment '#extra-extra'. These will be untouched.

 The script creates (overwrites) 2 files in its directory:
 1. changes.txt ... contains list of added, changed and deleted strings
    between new and old version. 
 2. new-trans.rul ... merge of your translation with the new version

    * New strings are added.
    * Removed strings are removed, regardles of their translation.
    * Unchanged strings remain unchanged.
    * Changed non-translated strings are changed to new version.
    * Changed translated strings: part of changed string is appended
      to them with ## prefix

"""
import io, re
from collections import OrderedDict
from difflib import SequenceMatcher
#pocitadlo kroku
global KROKU
KROKU = 10

#hlaseni o prubehu
def hlaska(hlaska):
    global KROKU
    print("{0}. {1}".format(KROKU , hlaska))
    KROKU -= 1

# dekodujRadek vrati pole velikosti:
#1 pro nestringove
#4 pro string, kde 0 nic, 1 strID, 2 string, 3 konec radku
def dekodujRadek(radek):
    #priprava regexp
    vzorRE = '^ {6}([^:]+)[:](.*)$' #6 mezer, strID, text
    return re.split(vzorRE, radek) 

# udelejYamlSlovnik nacte do dictionary obsah yaml souboru
def udelejYamlSlovnik(soubor):
    slovnik = OrderedDict({})
    sklonovani = False #zda se jedna o sekci se sklonovanim
    nazevSklon = '' #strID sklonovani
    obsahSklon = ''

    with io.open(soubor, 'r', encoding='utf8') as zdrojSoubor:
        for line in zdrojSoubor:
            zdrojPole = dekodujRadek(line) #rozsekej radek
            if (len(zdrojPole) == 1): #neni string
                slovnik[zdrojPole[0]] = ''
            elif (len(zdrojPole) == 4): #preklad string
                if (zdrojPole[2] not in ['', ' '] and not sklonovani): #obycejny string s hodnotou a nesklonujeme
                    slovnik[zdrojPole[1]] = zdrojPole[2].strip()
                elif (zdrojPole[2] in ['', ' '] and not sklonovani): #zacatek sklonovani
                    sklonovani = True
                    nazevSklon = zdrojPole[1]
                    obsahSklon = ''
                elif (sklonovani): #prubeh sklonovani
                    if (zdrojPole[1][:2] == '  '): #radek sklonovani
                        obsahSklon += '\n      ' + zdrojPole[1] + ':' + zdrojPole[2]
                    elif (zdrojPole[2] == ''): #sklon ukonceno zacatkem noveho
                        slovnik[nazevSklon] = obsahSklon #zapis vsechny radky sklonovani
                        #zacni nove
                        nazevSklon = zdrojPole[1]
                        obsahSklon = '\n'
                    elif (zdrojPole[2] != ''): #sklon ukoncen obyc stringem
                        sklonovani = False
                        slovnik[nazevSklon] = obsahSklon #zapis vsechny radky sklonovani
                        #ukonci a pak
                        nazevSklon = ''
                        obsahSklon = ''
                        slovnik[zdrojPole[1]] = zdrojPole[2].strip()
                    else: #chyba
                        print('sklonuje se, ale neni to ani prubeh, ani konec sklonovani')
                        quit()
                else: #chyba
                    print('nejde o normal a ani o sklonovani')
                    quit()
            else: #chyba
                print('Pole řádku nemá ani 1, ani 4 položky')
                quit()
    return slovnik

# vrati cast noveho textu, ktery neni ve starem
def vratRozdil(stary, novy):
    if (stary > novy): #rozdil delej z delsiho retezce
        delta=stary
    else:
        delta=novy
    vrat = '' #navratovy text
    s = SequenceMatcher(None, stary, novy)
    i = 9999 #delka retezce
    for block in s.get_matching_blocks():
        if (len(novy[block.size:]) < i): #a vracej kratsi z rozdilu
            vrat = novy[block.size:]
            i = len(novy[block.size:])
    return vrat

#nacti ze souboru do slovniku
try:
    hlaska('loading old.rul')
    starySlovnik = udelejYamlSlovnik('old.rul')
    hlaska('loading new.rul')
    novySlovnik = udelejYamlSlovnik('new.rul')
    hlaska('loading tra.rul')
    prelozenySlovnik = udelejYamlSlovnik('tra.rul')
except FileNotFoundError:
    print('You need these files in this script directory: new.rul, old.rul, tra.rul')
    quit()
hlaska('files loaded')

#mazani klicu ze stareho a prelozeneho, ktere nejsou v nove verzi
hlaska('removing old keys')
mazatSet = starySlovnik.keys() - novySlovnik.keys()
for maz in mazatSet:
    del starySlovnik[maz]
    if (maz in prelozenySlovnik.keys()):
        del prelozenySlovnik[maz]

#zachovani extra stringu
hlaska('storing extra strings')
extraSlovnik = OrderedDict({})
nalezenoExtra = False #narazil na sekci?
for strid, retezec in prelozenySlovnik.items():
    if (nalezenoExtra or '#extra-extra' in strid):
        extraSlovnik[strid] = retezec
        nalezenoExtra = True

#nalezeni identifikatoru jazyka
jazykId = '  - type: en-US\n'
for klic in prelozenySlovnik.keys():
    if ('  - type: ' in klic):
        jazykId = klic
        break

#pridani novych klicu do prelozeneho slovniku, ve kterem uz nejsou vymazane zaznamy
pridatSet = novySlovnik.keys() - starySlovnik.keys() #set novych zaznamu
docasnySlovnik = OrderedDict({}) #zde bude kombinace noveho a prelozeneho
seznamZmen = [] #klice, ktere byly prelozeny a jsou nove
jazykPrepinac = True; #bude zapsan jazyk

hlaska('merging new ver with translation')
for klic, hodnota in novySlovnik.items():
    if (klic in pridatSet): #tenhle je novy zaznam
        docasnySlovnik[klic] = hodnota #pridej bez zmeny
    else: #jinak je stary zaznam
        if (jazykPrepinac and '  - type:' in klic): #identifikator jazyka
            jazykPrepinac = False #uz to nebudu kontrolovat
            docasnySlovnik[jazykId] = hodnota #nastav spravny jazyk
        elif (hodnota == ''): #koment nebo jiny
            docasnySlovnik[klic] = '' #zapis co tam je
        elif (hodnota == starySlovnik[klic]): #string se nemenil
            docasnySlovnik[klic] = prelozenySlovnik[klic] #tak ho nemen
        elif (hodnota != starySlovnik[klic] == prelozenySlovnik[klic]): #zmenil se neprelozeny string
            docasnySlovnik[klic] = hodnota #nahrad ho novou verzi
        else: #zmenil se string a byl prelozeny
            if (prelozenySlovnik[klic][-1] == '"'): #pores text v uvozovkach
                docasnySlovnik[klic] = prelozenySlovnik[klic][:-1]+' ##'+ re.sub('(\")', '', vratRozdil(starySlovnik[klic], hodnota))+'"' #pridej rozdil en stare a nove do komentare za preklad
            else:
                docasnySlovnik[klic] = prelozenySlovnik[klic]+' ##'+ vratRozdil(starySlovnik[klic], hodnota) #pridej rozdil en stare a nove do komentare za preklad
            seznamZmen.append(klic)

hlaska('counting % of translation')
#pocitej procenta prekladu
celkemIds = 0
prelozenoIds = 0
for klic, hodnota in novySlovnik.items():
    if (hodnota != ''):
        celkemIds += 1
        if (docasnySlovnik[klic] != hodnota):
            prelozenoIds += 1
            
#pridej vytvorenemu docasnemu extra zaznamy
docasnySlovnik.update(extraSlovnik)

#zapisovac zmen do souboru
def zapisZmeny(soubor, titulek, seznam):
    if (len(seznam) > 0):
        soubor.write(titulek+'\n')
        for zmena in seznam:
            soubor.write(zmena+"\n")
        soubor.write('---\n')

#vytvor info soubor se zmenami
hlaska('creating changes.txt')
with io.open('changes.txt', 'w', encoding='utf8') as zmenyS:
    zapisZmeny(zmenyS, 'Numbers:', ["Total IDs: " + str(celkemIds), "Translated: " + str(prelozenoIds), "Done " + str((prelozenoIds / celkemIds) * 100) + " percent"])
    zapisZmeny(zmenyS, 'Added keys:', pridatSet)
    zapisZmeny(zmenyS, 'Changed keys:', seznamZmen)
    zapisZmeny(zmenyS, 'Removed keys:', mazatSet)

#vytvor novy soubor prekladu
hlaska('creating new-trans.rul')
with io.open('new-trans.rul', 'w', encoding='utf8') as novyS:
    for strid, retezec in docasnySlovnik.items():
        if (retezec == ''): #nejedna se o retezec
            novyS.write(strid) #zapis jen hodnotu
        else:
            novyS.write('      '+strid+': '+retezec+'\n') #cely zaznam
            
#konec programu
hlaska('END OF PROGRAM')
